
package org.petctviewer.anonymize;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import com.github.stephenc.javaisotools.eltorito.impl.ElToritoConfig;
import com.github.stephenc.javaisotools.iso9660.ConfigException;
import com.github.stephenc.javaisotools.iso9660.ISO9660RootDirectory;
import com.github.stephenc.javaisotools.iso9660.impl.CreateISO;
import com.github.stephenc.javaisotools.iso9660.impl.ISO9660Config;
import com.github.stephenc.javaisotools.iso9660.impl.ISOImageFileHandler;
import com.github.stephenc.javaisotools.joliet.impl.JolietConfig;
import com.github.stephenc.javaisotools.rockridge.impl.RockRidgeConfig;
import com.github.stephenc.javaisotools.sabre.HandlerException;
import com.github.stephenc.javaisotools.sabre.StreamHandler;

//EFFACEMENT DES TEMP DIRECTORY A GERER CF PROGRAMME MAXIME
public class ZipAndViewer {
	File zipDicom;
	File viewerPackage ;
	Path folderTempImage;
	Path folderTempViewer;
	Path tempFolder;
	File destination;
	
	public ZipAndViewer(File zipDicom, File destination, File viewerPackage) throws IOException {
		this.zipDicom=zipDicom;
		this.destination=destination;
		this.viewerPackage=viewerPackage;
		
	}
	
	public void unzip() {
		
		try {
			tempFolder = Files.createTempDirectory("ISO_");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		unzip(zipDicom, tempFolder);
		unzip(viewerPackage, tempFolder);
	}
	
	
	public void ZipAndViewerToZip() throws IOException {
		try {
			destination.createNewFile();
			ZipOutputStream outStream = new ZipOutputStream(new FileOutputStream(destination));
			readZip(outStream , viewerPackage.toString());
			readZip(outStream , zipDicom.toString());
			outStream.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	private void unzip(File zip, Path tempFolder){
		 try {
		     byte[] buffer = new byte[1024];
	    
	    	//get the zip file content
	    	ZipInputStream zis;
			zis = new ZipInputStream(new FileInputStream(zip));
			
	    	//get the zipped file list entry
	    	ZipEntry ze = zis.getNextEntry();
	    	
	    	//textArea.append("Unzipping ");
	    	
	    	while(ze!=null){
	     	   	String fileName = ze.getName();
	     	    
	            File newFile = new File(tempFolder + File.separator + fileName);
	            
	            if (ze.isDirectory()) {
	         	// if the entry is a directory, make the directory
	                newFile.mkdirs();
	            }
	            
	            else {
	         	    new File(newFile.getParent()).mkdirs();
	                 //create all non exists folders
	                 //else you will hit FileNotFoundException for compressed folder
	                
	
	                 FileOutputStream fos = new FileOutputStream(newFile);
	
	                 int len;
	                 while ((len = zis.read(buffer)) > 0) {
	            		fos.write(buffer, 0, len);
	                 }
	
	                 fos.close();
	                 
	            }
	            ze = zis.getNextEntry();
	     	}

	        zis.closeEntry();
	    	zis.close();
	    	recursiveDeleteOnExit(tempFolder);
	     } catch (IOException e) {
				e.printStackTrace();
			}
	     
	     
	}
	
	// Concatenation de ZIP
	private void readZip(ZipOutputStream outStream, String targetFile) throws Exception {
	    ZipInputStream inStream = new ZipInputStream(new FileInputStream(targetFile));
	    byte[] buffer = new byte[1024];
	    int len = 0;

	    // A DEBEUGER
	    for (ZipEntry e; (e = inStream.getNextEntry()) != null;) {
	        ZipEntry destEntry =  new ZipEntry (e.getName());
	        
	    	outStream.putNextEntry(destEntry);
	        while ((len = inStream.read(buffer)) > 0) {
	            outStream.write(buffer, 0, len);
	        }
	    }
	    
	    inStream.close();
	   
	}
	
	
	
	public void generateIsoFile() {
		// from https://github.com/stephenc/java-iso-tools/blob/master/iso9660-writer/ISOtest.java
		try {
		// Directory hierarchy, starting from the root
		ISO9660RootDirectory.MOVED_DIRECTORIES_STORE_NAME = "rr_moved";
		ISO9660RootDirectory root = new ISO9660RootDirectory();
		
		root.addContentsRecursively(tempFolder.toFile());
		
		
		// ISO9660 support
		ISO9660Config iso9660Config = new ISO9660Config();
		iso9660Config.allowASCII(false);
		iso9660Config.setInterchangeLevel(1);
		iso9660Config.restrictDirDepthTo8(true);
		iso9660Config.setPublisher("petctviewer.org");
		iso9660Config.setVolumeID("Patient Images");
		iso9660Config.setDataPreparer("Salim Kanoun");
		//A mettre
		//iso9660Config.setCopyrightFile(new File("Copyright.txt"));
		iso9660Config.forceDotDelimiter(true);

		RockRidgeConfig rrConfig = null;
		// Rock Ridge support
		rrConfig = new RockRidgeConfig();
		rrConfig.setMkisofsCompatibility(false);
		rrConfig.hideMovedDirectoriesStore(true);
		rrConfig.forcePortableFilenameCharacterSet(true);
		

		JolietConfig jolietConfig = null;
		// Joliet support
		jolietConfig = new JolietConfig();
		jolietConfig.setPublisher("petctviewer.org");
		jolietConfig.setVolumeID("Patient Images");
		jolietConfig.setDataPreparer("Salim Kanoun");
		// A mettre
		//jolietConfig.setCopyrightFile(new File("Copyright.txt"));
		jolietConfig.forceDotDelimiter(true);
		
		//Pas besoin de secteur bootable
		ElToritoConfig elToritoConfig = null;

		// Create ISO
		StreamHandler streamHandler = new ISOImageFileHandler(destination);
		CreateISO iso = new CreateISO(streamHandler, root);
		iso.process(iso9660Config, rrConfig, jolietConfig, elToritoConfig);
		
		} 
		catch (HandlerException |FileNotFoundException |ConfigException e) {e.printStackTrace();}
		
		
		
		
	}
	
	
	
	private static void recursiveDeleteOnExit(Path path) throws IOException {
		  Files.walkFileTree(path, new SimpleFileVisitor<Path>() {
		    @Override
		    public FileVisitResult visitFile(Path file,
		        @SuppressWarnings("unused") BasicFileAttributes attrs) {
		      file.toFile().deleteOnExit();
		      return FileVisitResult.CONTINUE;
		    }
		    @Override
		    public FileVisitResult preVisitDirectory(Path dir,
		        @SuppressWarnings("unused") BasicFileAttributes attrs) {
		      dir.toFile().deleteOnExit();
		      return FileVisitResult.CONTINUE;
		    }
		  });
		}

}
